unit MainUnit;

interface

procedure Main;

implementation

uses
  Windows,
  SysUtils, Classes,
  PTLPorts;

function InStrC(C: Char; S: String; StartIndex: Integer = 1): LongWord;
var
  I: Integer;

begin
  for I := StartIndex to Length(S) do
    if S[I] = C then begin
      Result := I;
      Exit;
    end
  ;

  Result := 0;
end;

function SplitStr(S: String; SplitChar: Char; var Before, After: String): Integer;
begin
  Result := InStrC(SplitChar, S);
  if Result = 0 then begin
    Before := S;
    After  := '';
  end
  else begin
    Before := Copy(S, 1, Result - 1);
    After  := Copy(S, Result + 1, MAXINT);
  end;
end;



const
  VerStr = 'APTLConfig 1.0';

var
  PortModes: array of record
    PortID: LongInt;
    PortMode: LongWord;
  end;
  Port: array of WideString;
  Log: TStringList;
  MyConfigFile: String = 'APTLConfig.cfg';
  MyLogFile: String = 'APTLConfig.log';

function CodedDT(DT: TDateTime): String;
var
  Years, Months, Days, Hours, Minutes, Seconds, mSeconds: Word;

begin
  DecodeTime(DT, Hours, Minutes, Seconds, mSeconds);
  DecodeDate(DT, Years, Months, Days);
  Result := Format('%d:%.2d:%.2d::%d:%.2d:%.2d', [ Years, Months, Days, Hours, Minutes, Seconds ]);
end;

procedure FixupDefPaths;
var
  BasePath: String;

begin
  BasePath := ParamStr(0);
  BasePath := IncludeTrailingPathDelimiter(ExtractFilePath(BasePath));
  MyConfigFile := BasePath + MyConfigFile;
  MyLogFile := BasePath + MyLogFile;
end;

procedure GatherParams;
var
  I: Integer;
  Param: String;

begin
  for I := 1 to ParamCount do begin
    Param := ParamStr(I);
    if Length(Param) = 0 then Continue;
    if (Param[1] = '"') and (Param[Length(Param)] = '"') then Param := Copy(Param, 2, Length(Param) - 2);
    if Length(Param) = 0 then Continue;
    if Param[1] = '/' then begin
      if UpperCase(Copy(Param, 1, 5)) = '/LOG=' then
        MyLogFile := Copy(Param, 6, MAXINT)
      ;
    end
    else begin
      if FileExists(Param) then MyConfigFile := Param;
    end;
  end;
end;

function LoadModes: Boolean;
var
  Config: TStringList;
  I, J: Integer;
  LValue, RValue, URValue: String;
  PortID: LongInt;
  PortMode: LongWord;

begin
  Config := TStringList.Create;
  try
    Config.LoadFromFile(MyConfigFile);
  except
    on E: Exception do begin
      Log.Add('Error loading ' + MyConfigFile + ':');
      Log.Add(#9 + E.ClassName + ': ' + E.Message);
      Log.Add(#9'Aborting.');

      Result := False;
      Exit;
    end;
  end;

  SetLength(PortModes, 0);
  for I := 0 to Config.Count - 1 do begin
    PortID := -1;
    PortMode := $FFFFFFFF;

    LValue := Trim(Config[I]);
    if (Length(LValue) = 0) or (LValue[1] = '[') or (LValue[1] = ';') then Continue;

    SplitStr(LValue, '=', LValue, RValue);
    LValue := Trim(LValue);
    RValue := Trim(RValue);
    if UpperCase(Copy(LValue, 1, 4)) <> 'PORT' then begin
      Log.Add('LValue "' + LValue + '" on line ' + IntToStr(I + 1) + ' should be of "Port0", "Port1", etc. form.');
      Continue;
    end;
    try
      PortID := StrToInt(Copy(LValue, 5, MAXINT));
    except
      Log.Add('LValue "' + LValue + '" on line ' + IntToStr(I + 1) + ' should be of "Port0", "Port1", etc. form.');
      Continue;
    end;

    URValue := UpperCase(RValue);
    if URValue = 'RS232' then
      PortMode := $0
    else if URValue = 'RS422' then
      PortMode := $1
    else if URValue = 'RS485-4W' then
      PortMode := $B
    else if (URValue = 'RS485') or (URValue = 'RS485-2W') then
      PortMode := $F
    else begin
      Log.Add('RValue "' + LValue + '" on line ' + IntToStr(I + 1) + ' should be "RS232", "RS422", "RS485", "RS485-4W".');
      Continue;
    end;

    J := Length(PortModes);
    SetLength(PortModes, J + 1);
    PortModes[J].PortID := PortID;
    PortModes[J].PortMode := PortMode;
  end;

  Config.Free;
  Result := True;
end;

procedure DetectPorts;
var
  I, J: Integer;
  Nom: WideString;

begin
  I := 0;
  SetString(Nom, nil, 256);
  SetLength(Port, 0);
  repeat
    if not PTLPortEnum(I, @Nom[1], Length(Nom)) then Break;
    SetLength(Port, I + 1);

    J := 1;
    repeat
      if J > Length(Nom) then begin
        Port[I] := Nom;
        Break;
      end
      else if Nom[J] = #0 then begin
        Port[I] := Copy(Nom, 1, J - 1);
        Break;
      end;
      Inc(J);
    until False;

    Inc(I);
  until False;
  Nom := '';
end;

procedure SetPorts;
var
  I, J: Integer;
  BoardID, PortID: LongInt;

label
  EndForIf;

begin
  Log.Add(IntToStr(Length(Port)) + ' ports');

  for I := 0 to High(Port) do begin
    if not PTLPortGetLocation(@Port[I][1], @BoardID, @PortID) then Continue;

    Log.Add('Board ' + IntToStr(BoardID) + ', port ' + IntToStr(PortID));

    if BoardID <> 0 then Continue;

    for J := 0 to High(PortModes) do
      if PortModes[J].PortID = PortID then begin
        if not PTLPortTXVR(@Port[I][1], @PortModes[J].PortMode, True) then begin
          Log.Add('Failed to set mode for port ' + IntToStr(PortID) + '.');
        end;
        PortModes[J].PortID := -1;
        goto EndForIf;
      end
    ;//else
      Log.Add('Port ' + IntToStr(PortID) + ' found with no mode assigned.');
    EndForIf: ;
  end;
  for J := 0 to High(PortModes) do
    if PortModes[J].PortID <> -1 then begin
      Log.Add('Port ' + IntToStr(PortModes[J].PortID) + ' not found.');
    end
  ;
end;

procedure Main;
begin
  FixupDefPaths;
  GatherParams;

  DeleteFile(MyLogFile);
  Log := TStringList.Create;
  Log.Add(VerStr);
  Log.Add(CodedDT(Now));

  if LoadModes then begin
    DetectPorts;
    SetPorts;
  end;

  try
    Log.SaveToFile(MyLogFile);
  except
  end;
  Log.Free;
end;

end.
